@@ -1,160 +1,81 @@
-import time
-import copy
-import json
+import json, time, platform
 from hashlib import sha256
-import binascii
-import platform
-from typing import Dict, Union, cast
+from typing import Dict, Sequence, Tuple, Union
 
 import requests
-import OpenSSL
-import cryptography
 
-from . import __version__ as sewer_version
+from .auth import ChalListType, ErrataListType, ProviderBase
 from .config import ACME_DIRECTORY_URL_PRODUCTION
-from .lib import create_logger, log_response, safe_base64
+from .crypto import AcmeCsr, AcmeKey, AcmeAccount
+from .lib import create_logger, log_response, safe_base64, sewer_meta, AcmeRegistrationError
 
 
 class Client:
     """
-    todo: improve documentation.
-
-    usage:
-        import sewer
-        provider = sewer.CloudFlareDns(
-            CLOUDFLARE_EMAIL='example@example.com',
-            CLOUDFLARE_API_KEY='nsa-grade-api-key'
-        )
-
-        ### to create a new certificate.
-        client = sewer.Client(
-            domain_name='example.com', provider=provider
-        )
-        certificate = client.cert()
-        certificate_key = client.certificate_key
-        account_key = client.account_key
-
-        with open('certificate.crt', 'w') as certificate_file:
-            certificate_file.write(certificate)
-
-        with open('certificate.key', 'w') as certificate_key_file:
-            certificate_key_file.write(certificate_key)
-
-        ### to renew a certificate:
-        with open('account_key.key', 'r') as account_key_file:
-            account_key = account_key_file.read()
-
-        client = sewer.Client(
-            domain_name='example.com', provider=provider,
-            account_key=account_key
-        )
-        certificate = client.renew()
-        certificate_key = client.certificate_key
-
-    todo: handle more exceptions
+    refer to docs/sewer-as-a-library for usage, etc.
     """
 
     def __init__(
         self,
-        domain_name,
-        dns_class=None,
-        domain_alt_names=None,
-        contact_email=None,
-        account_key=None,
-        certificate_key=None,
-        bits=2048,
-        digest="sha256",
-        provider=None,
-        ACME_REQUEST_TIMEOUT=7,
-        ACME_AUTH_STATUS_WAIT_PERIOD=8,
-        ACME_AUTH_STATUS_MAX_CHECKS=3,
-        ACME_DIRECTORY_URL=ACME_DIRECTORY_URL_PRODUCTION,
-        ACME_VERIFY=True,
-        LOG_LEVEL="INFO",
+        *,
+        domain_name: str,
+        account: AcmeAccount,
+        cert_key: AcmeKey,
+        is_new_acct=False,
+        dns_class: ProviderBase = None,
+        domain_alt_names: Sequence[str] = None,
+        contact_email: str = None,
+        provider: ProviderBase = None,
+        ACME_REQUEST_TIMEOUT: int = 7,
+        ACME_AUTH_STATUS_WAIT_PERIOD: int = 8,
+        ACME_AUTH_STATUS_MAX_CHECKS: int = 3,
+        ACME_DIRECTORY_URL: str = ACME_DIRECTORY_URL_PRODUCTION,
+        ACME_VERIFY: bool = True,
+        LOG_LEVEL: str = "INFO",
     ):
-        """
-        :param domain_name:                  (required) [string]
-            the name that you want to acquire/renew certificate for. wildcards are allowed.
-        :param dns_class:                    (required) [class]
-            (DEPRECATED) a subclass of BaseDns which will be called to create/delete DNS TXT records.
-            do not pass this parameter if also passing provider.
-        :param provider:                (required) [class]
-            a subclass of ProviderBase which will be called to create/delete auth records.
-            do not pass this parameter if also passing dns_class
-        :param domain_alt_names:             (optional) [list]
-            list of alternative names that you want to be bundled into the same certificate as domain_name.
-        :param contact_email:                (optional) [string]
-            a contact email address
-        :param account_key:                  (optional) [string]
-            a string whose contents is an ssl certificate that identifies your account on the acme server.
-            if you do not provide one, this client will issue a new certificate else will renew.
-        :param certificate_key:              (optional) [string]
-            a string whose contents is a private key that will be incorporated into your new certificate.
-            if you do not provide one, this client will issue a new certificate else will renew.
-        :param bits:                         (optional) [integer]
-            number of bits that will be used to create your certificates' private key.
-        :param digest:                       (optional) [string]
-            the ssl digest type to be used in signing the certificate signing request(csr)
-        :param ACME_REQUEST_TIMEOUT:         (optional) [integer]
-            the max time that the client will wait for a network call to complete.
-        :param ACME_AUTH_STATUS_WAIT_PERIOD: (optional) [integer]
-            the interval between two consecutive client polls on the acme server to check on authorization status
-        :param ACME_AUTH_STATUS_MAX_CHECKS:  (optional) [integer]
-            the max number of times the client will poll the acme server to check on authorization status
-        :param ACME_DIRECTORY_URL:           (optional) [string]
-            the url of the acme servers' directory endpoint
-        :param ACME_VERIFY:                  (optional) [bool]
-            suppress verification of SSL cert when set to False (for pebble); hint: -Wignore
-        :param LOG_LEVEL:                    (optional) [string]
-            the level to output log messages at. one of; 'DEBUG', 'INFO', 'WARNING', 'ERROR' or 'CRITICAL'
-        """
+
+        ### do some type checking of some parameters
+
+        ### FIX ME ### spotty and not always complete; some should raise TypeError, not ValueError
 
         if not isinstance(domain_alt_names, (type(None), list)):
             raise ValueError(
-                """domain_alt_names should be of type:: None or list. You entered {0}""".format(
-                    type(domain_alt_names)
-                )
-            )
-        elif not isinstance(contact_email, (type(None), str)):
-            raise ValueError(
-                """contact_email should be of type:: None or str. You entered {0}""".format(
-                    type(contact_email)
-                )
-            )
-        elif not isinstance(account_key, (type(None), str)):
-            raise ValueError(
-                """account_key should be of type:: None or str. You entered {0}.
-                More specifically, account_key should be the result of reading an ssl account certificate""".format(
-                    type(account_key)
-                )
-            )
-        elif not isinstance(certificate_key, (type(None), str)):
-            raise ValueError(
-                """certificate_key should be of type:: None or str. You entered {0}.
-                More specifically, certificate_key should be the result of reading an ssl certificate""".format(
-                    type(certificate_key)
-                )
-            )
-        elif LOG_LEVEL.upper() not in ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]:
-            raise ValueError(
-                """LOG_LEVEL should be one of; 'DEBUG', 'INFO', 'WARNING', 'ERROR' or 'CRITICAL'. not {0}""".format(
-                    LOG_LEVEL
-                )
-            )
-        elif dns_class is not None and provider is not None:
-            raise ValueError(
-                "passed both the DEPRECATED 'dns_class' parameter as well as 'provider'."
+                "domain_alt_names should be None or a list of strings, not %s" % domain_alt_names
             )
 
+        if not isinstance(contact_email, (type(None), str)):
+            raise ValueError("contact_email should be None or a string, not %s" % contact_email)
+
+        if LOG_LEVEL.upper() not in ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]:
+            raise ValueError(
+                "LOG_LEVEL must be one of 'DEBUG', 'INFO', 'WARNING', 'ERROR' or 'CRITICAL'"
+            )
+
+        if dns_class is not None and provider is not None:
+            raise ValueError(
+                "Client was passed both the DEPRECATED dns_class argument and provider."
+            )
+
+        if not isinstance(account, AcmeAccount):
+            raise TypeError("The account argument must be an AcmeAccount.")
+
+        if not isinstance(cert_key, AcmeKey):
+            raise TypeError("The argument cert_key must be an AcmeKey.")
+
+        ### setup Client's global variables
+
         self.domain_name = domain_name
-        self.provider = provider if provider is not None else dns_class
+
+        # long winded is both stricter check as well as giving mypy a clear enough hint
+        if isinstance(provider, ProviderBase):
+            self.provider = provider
+        elif isinstance(dns_class, ProviderBase):
+            self.provider = dns_class
+
         if not domain_alt_names:
             domain_alt_names = []
-        self.domain_alt_names = domain_alt_names
-        self.domain_alt_names = list(set(self.domain_alt_names))
+        self.domain_alt_names = list(set(domain_alt_names))
         self.contact_email = contact_email
-        self.bits = bits
-        self.digest = digest
         self.ACME_REQUEST_TIMEOUT = ACME_REQUEST_TIMEOUT
         self.ACME_AUTH_STATUS_WAIT_PERIOD = ACME_AUTH_STATUS_WAIT_PERIOD
         self.ACME_AUTH_STATUS_MAX_CHECKS = ACME_AUTH_STATUS_MAX_CHECKS
@@ -162,13 +83,14 @@ class Client:
         self.ACME_VERIFY = ACME_VERIFY
         self.LOG_LEVEL = LOG_LEVEL.upper()
 
+        self.account = account
+        self.cert_key = cert_key
+        self.is_new_acct = is_new_acct
+
         self.logger = create_logger(__name__, LOG_LEVEL)
 
         try:
-            self.all_domain_names = copy.copy(self.domain_alt_names)
-            self.all_domain_names.insert(0, self.domain_name)
-            self.domain_alt_names = list(set(self.domain_alt_names))
-
+            self.all_domain_names = [self.domain_name] + self.domain_alt_names
             self.User_Agent = self.get_user_agent()
             acme_endpoints = self.get_acme_endpoints().json()
             self.ACME_GET_NONCE_URL = acme_endpoints["newNonce"]
@@ -178,19 +100,7 @@ class Client:
             self.ACME_NEW_ORDER_URL = acme_endpoints["newOrder"]
             self.ACME_REVOKE_CERT_URL = acme_endpoints["revokeCert"]
 
-            # unique account identifier
-            # https://tools.ietf.org/html/draft-ietf-acme-acme#section-6.2
-            self.kid = None
-
-            self.certificate_key = certificate_key or self.create_certificate_key()
-            self.csr = self.create_csr()
-
-            if not account_key:
-                self.account_key = self.create_account_key()
-                self.PRIOR_REGISTERED = False
-            else:
-                self.account_key = account_key
-                self.PRIOR_REGISTERED = True
+            self.acme_csr = AcmeCsr(cn=domain_name, san=domain_alt_names, key=self.cert_key)
 
             if dns_class is not None:
                 self.logger.warning(
@@ -199,7 +109,7 @@ class Client:
 
             self.logger.info(
                 "intialise_success, sewer_version={0}, domain_names={1}, acme_server={2}".format(
-                    sewer_version.__version__,
+                    sewer_meta("version"),
                     self.all_domain_names,
                     self.ACME_DIRECTORY_URL[:20] + "...",
                 )
@@ -211,7 +121,7 @@ class Client:
             self.logger.error("Unable to intialise Client. error={0}".format(str(e)[:100]))
             raise e
 
-    def GET(self, url: str, **kwargs: Union[str, Dict[str, str], bool]) -> requests.Response:
+    def GET(self, url: str) -> requests.Response:
         """
         wrap requests.get (and post and head, below) to allow:
           * injection of e.g. UserAgent header in one place rather than all over
@@ -219,32 +129,34 @@ class Client:
           * paves the way to inject the verify option, required to use pebble
         """
 
-        return self._request("GET", url, **kwargs)
+        return self._request("GET", url)
 
-    def HEAD(self, url: str, **kwargs: Union[str, Dict[str, str], bool]) -> requests.Response:
-        return self._request("HEAD", url, **kwargs)
+    # HEAD is still waiting for the test rewrite to let it be used... very low priority :-(
+    def HEAD(self, url: str) -> requests.Response:
+        return self._request("HEAD", url)
 
-    def POST(self, url: str, **kwargs: Union[str, Dict[str, str], bool]) -> requests.Response:
-        return self._request("POST", url, **kwargs)
+    def POST(
+        self, url: str, *, data: bytes = None, headers: Dict[str, str] = None
+    ) -> requests.Response:
+        return self._request("POST", url, data=data, headers=headers)
 
     def _request(
-        self, method: str, url: str, **kwargs: Union[str, Dict[str, str], bool]
+        self, method: str, url: str, *, data: bytes = None, headers: Dict[str, str] = None
     ) -> requests.Response:
         """
         shared implementation for GET, POST and HEAD
-        * injects standard request options unless they are already given in kwargs
+        * injects standard request options unless they are already given in headers
           * header:UserAgent, timeout
           * verify - this is a hack to make sewer accept pebble's intentionally bogus cert
         """
 
-        # if there's no UserAgent header in args, inject it
-        headers = cast(Dict[str, str], kwargs.setdefault("headers", {}))
+        if headers is None:
+            headers = {}
+
         if "UserAgent" not in headers:
             headers["UserAgent"] = self.User_Agent
 
-        # add standard timeout if there's none already present
-        if "timeout" not in kwargs:
-            kwargs["timeout"] = self.ACME_REQUEST_TIMEOUT
+        kwargs = {"timeout": self.ACME_REQUEST_TIMEOUT}  # type: Dict[str, Union[str, int]]
 
         ### FIX ME ### can get current bogus cert from pebble, figure out how to use it here?
 
@@ -252,16 +164,17 @@ class Client:
         if not self.ACME_VERIFY:
             kwargs["verify"] = False
 
+        # this is what we'd do if damn near every test didn't mock requests.{get,post}
+        # response = requests.request(method, url, headers=headers, **kwargs)
+
         # awkward implementation to maintain compatibility with current mocked tests
         if method == "GET":
-            ### Incredibly stupid hack to reconcile mypy to requests' API
-            response = requests.get(url, None, **kwargs)
+            # mypy seems to be confused if params isn't explicitly passed, wtf?
+            response = requests.get(url, params=None, headers=headers, **kwargs)
         elif method == "HEAD":
-            response = requests.head(url, **kwargs)
+            response = requests.head(url, headers=headers, **kwargs)
         elif method == "POST":
-            ### Incredibly stupid hack to reconcile mypy to requests' API
-            data_hack = cast(Union[Dict[str, str], None], kwargs.pop("data", None))
-            response = requests.post(url, data_hack, **kwargs)
+            response = requests.post(url, data, headers=headers, **kwargs)
 
         return response
 
@@ -271,8 +184,8 @@ class Client:
             requests_version=requests.__version__,
             system=platform.system(),
             machine=platform.machine(),
-            sewer_version=sewer_version.__version__,
-            sewer_url=sewer_version.__url__,
+            sewer_version=sewer_meta("version"),
+            sewer_url=sewer_meta("url"),
         )
 
     def get_acme_endpoints(self):
@@ -290,70 +203,17 @@ class Client:
             )
         return get_acme_endpoints
 
-    def create_certificate_key(self):
-        self.logger.debug("create_certificate_key")
-        return self.create_key().decode()
-
-    def create_account_key(self):
-        self.logger.debug("create_account_key")
-        return self.create_key().decode()
-
-    def create_key(self, key_type=OpenSSL.crypto.TYPE_RSA):
-        key = OpenSSL.crypto.PKey()
-        key.generate_key(key_type, self.bits)
-        private_key = OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, key)
-        return private_key
-
-    def create_csr(self):
-        """
-        https://tools.ietf.org/html/draft-ietf-acme-acme#section-7.4
-        The CSR is sent in the base64url-encoded version of the DER format. (NB: this
-        field uses base64url, and does not include headers, it is different from PEM.)
-        """
-        self.logger.debug("create_csr")
-        X509Req = OpenSSL.crypto.X509Req()
-        X509Req.get_subject().CN = self.domain_name
-
-        if self.domain_alt_names:
-            SAN = "DNS:{0}, ".format(self.domain_name).encode("utf8") + ", ".join(
-                "DNS:" + i for i in self.domain_alt_names
-            ).encode("utf8")
-        else:
-            SAN = "DNS:{0}".format(self.domain_name).encode("utf8")
-
-        X509Req.add_extensions(
-            [
-                OpenSSL.crypto.X509Extension(
-                    "subjectAltName".encode("utf8"), critical=False, value=SAN
-                )
-            ]
-        )
-        pk = OpenSSL.crypto.load_privatekey(
-            OpenSSL.crypto.FILETYPE_PEM, self.certificate_key.encode()
-        )
-        X509Req.set_pubkey(pk)
-        X509Req.set_version(2)
-        X509Req.sign(pk, self.digest)
-        return OpenSSL.crypto.dump_certificate_request(OpenSSL.crypto.FILETYPE_ASN1, X509Req)
+    ### FIX ME ### this is a kludge to fix Alec's needs until there's time to do the Acme* refactor
 
     def acme_register(self):
-        """
-        https://tools.ietf.org/html/draft-ietf-acme-acme#section-7.3
-        The server creates an account and stores the public key used to
-        verify the JWS (i.e., the "jwk" element of the JWS header) to
-        authenticate future requests from the account.
-        The server returns this account object in a 201 (Created) response, with the account URL
-        in a Location header field.
-        This account URL will be used in subsequest requests to ACME, as the "kid" value in the acme header.
-        If the server already has an account registered with the provided
-        account key, then it MUST return a response with a 200 (OK) status
-        code and provide the URL of that account in the Location header field.
-        If there is an existing account with the new key
-        provided, then the server SHOULD use status code 409 (Conflict) and
-        provide the URL of that account in the Location header field
-        """
-        self.logger.info("acme_register (newAccount)")
-        if self.PRIOR_REGISTERED:
+
+        self.logger.info("acme_register%s" % " (is new account)" if self.is_new_acct else "")
+
+        if self.account.has_kid():
+            self.logger.info("acme_register: key was already registered")
+            return None
+
+        if not self.is_new_acct:
             payload = {"onlyReturnExisting": True}
         elif self.contact_email:
             payload = {
@@ -364,28 +224,26 @@ class Client:
             payload = {"termsOfServiceAgreed": True}
 
         url = self.ACME_NEW_ACCOUNT_URL
-        acme_register_response = self.make_signed_acme_request(
+        response = self.make_signed_acme_request(
             url=url, payload=json.dumps(payload), needs_jwk=True
         )
         self.logger.debug(
-            "acme_register_response. status_code={0}. response={1}".format(
-                acme_register_response.status_code, log_response(acme_register_response)
+            "response. status_code={0}. response={1}".format(
+                response.status_code, log_response(response)
             )
         )
 
-        if acme_register_response.status_code not in [201, 200, 409]:
-            raise ValueError(
+        if response.status_code not in [201, 200, 409]:
+            raise AcmeRegistrationError(
                 "Error while registering: status_code={status_code} response={response}".format(
-                    status_code=acme_register_response.status_code,
-                    response=log_response(acme_register_response),
+                    status_code=response.status_code, response=log_response(response),
                 )
             )
 
-        kid = acme_register_response.headers["Location"]
-        setattr(self, "kid", kid)
+        self.account.set_kid(response.headers["Location"])
 
         self.logger.info("acme_register_success")
-        return acme_register_response
+        return response
 
     def apply_for_cert_issuance(self):
         """
@@ -490,7 +348,7 @@ class Client:
 
     def get_keyauthorization(self, token):
         self.logger.debug("get_keyauthorization")
-        acme_header_jwk_json = json.dumps(self.get_jwk(), sort_keys=True, separators=(",", ":"))
+        acme_header_jwk_json = json.dumps(self.account.jwk(), sort_keys=True, separators=(",", ":"))
         acme_thumbprint = safe_base64(sha256(acme_header_jwk_json.encode("utf8")).digest())
         acme_keyauthorization = "{0}.{1}".format(token, acme_thumbprint)
 
@@ -511,20 +369,17 @@ class Client:
         The server MUST provide information about its retry state to the
         client via the "errors" field in the challenge and the Retry-After
         """
-        self.logger.info("check_authorization_status")
+        self.logger.debug("check_authorization_status")
         desired_status = desired_status or ["pending", "valid"]
         number_of_checks = 0
         while True:
             time.sleep(self.ACME_AUTH_STATUS_WAIT_PERIOD)
-            check_authorization_status_response = self.make_signed_acme_request(
-                authorization_url, payload=""
-            )
-            authorization_status = check_authorization_status_response.json()["status"]
+            response = self.make_signed_acme_request(authorization_url, payload="")
+            authorization_status = response.json()["status"]
             number_of_checks = number_of_checks + 1
             self.logger.debug(
-                "check_authorization_status_response. status_code={0}. response={1}".format(
-                    check_authorization_status_response.status_code,
-                    log_response(check_authorization_status_response),
+                "response. status_code={0}. response={1}".format(
+                    response.status_code, log_response(response),
                 )
             )
             if authorization_status in desired_status:
@@ -538,8 +393,8 @@ class Client:
                     )
                 )
 
-        self.logger.info("check_authorization_status_success")
-        return check_authorization_status_response
+        self.logger.debug("check_authorization_status_success")
+        return response
 
     def respond_to_challenge(self, acme_keyauthorization, challenge_url):
         """
@@ -585,7 +440,7 @@ class Client:
         GET request to the order resource to obtain its current state.
         """
         self.logger.info("send_csr")
-        payload = {"csr": safe_base64(self.csr)}
+        payload = {"csr": safe_base64(self.acme_csr.public_bytes())}
         send_csr_response = self.make_signed_acme_request(
             url=finalize_url, payload=json.dumps(payload)
         )
@@ -627,11 +482,6 @@ class Client:
         self.logger.info("download_certificate_success")
         return pem_certificate
 
-    def sign_message(self, message):
-        self.logger.debug("sign_message")
-        pk = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, self.account_key.encode())
-        return OpenSSL.crypto.sign(pk, message.encode("utf8"), self.digest)
-
     def get_nonce(self):
         """
         https://tools.ietf.org/html/draft-ietf-acme-acme#section-6.4
@@ -643,46 +493,6 @@ class Client:
         nonce = response.headers["Replay-Nonce"]
         return nonce
 
-    @staticmethod
-    def stringfy_items(payload):
-        """
-        method that takes a dictionary and then converts any keys or values
-        in that are of type bytes into unicode strings.
-        This is necessary esp if you want to then turn that dict into a json string.
-        """
-        if isinstance(payload, str):
-            return payload
-
-        for k, v in payload.items():
-            if isinstance(k, bytes):
-                k = k.decode("utf-8")
-            if isinstance(v, bytes):
-                v = v.decode("utf-8")
-            payload[k] = v
-        return payload
-
-    def get_jwk(self):
-        """
-        calculate the JSON Web Key (jwk) from self.account_key
-        """
-        private_key = cryptography.hazmat.primitives.serialization.load_pem_private_key(
-            self.account_key.encode(),
-            password=None,
-            backend=cryptography.hazmat.backends.default_backend(),
-        )
-        public_key_public_numbers = private_key.public_key().public_numbers()
-        # private key public exponent in hex format
-        exponent = "{0:x}".format(public_key_public_numbers.e)
-        exponent = "0{0}".format(exponent) if len(exponent) % 2 else exponent
-        # private key modulus in hex format
-        modulus = "{0:x}".format(public_key_public_numbers.n)
-        jwk = {
-            "kty": "RSA",
-            "e": safe_base64(binascii.unhexlify(exponent)),
-            "n": safe_base64(binascii.unhexlify(modulus)),
-        }
-        return jwk
-
     def get_acme_header(self, url, needs_jwk=False):
         """
         https://tools.ietf.org/html/draft-ietf-acme-acme#section-6.2
@@ -694,12 +504,13 @@ class Client:
         - "url"
         """
         self.logger.debug("get_acme_header")
-        header = {"alg": "RS256", "nonce": self.get_nonce(), "url": url}
+        header = {"alg": self.account.key_desc.jwk["alg"], "nonce": self.get_nonce(), "url": url}
 
         if needs_jwk:
-            header["jwk"] = self.get_jwk()
+            header["jwk"] = self.account.jwk()
         else:
-            header["kid"] = self.kid
+            header["kid"] = self.account.kid
+
         return header
 
     def make_signed_acme_request(self, url, payload, needs_jwk=False):
@@ -708,8 +519,10 @@ class Client:
         payload64 = safe_base64(payload)
         protected = self.get_acme_header(url, needs_jwk)
         protected64 = safe_base64(json.dumps(protected))
-        signature = self.sign_message(message="{0}.{1}".format(protected64, payload64))  # bytes
-        signature64 = safe_base64(signature)  # str
+        message = ("%s.%s" % (protected64, payload64)).encode("utf-8")
+        #        signature = self.sign_message(message="{0}.{1}".format(protected64, payload64))  # bytes
+        #        signature64 = safe_base64(signature)  # str
+        signature64 = safe_base64(self.account.sign_message(message))
         data = json.dumps(
             {"protected": protected64, "payload": payload64, "signature": signature64}
         )
@@ -745,7 +558,7 @@ class Client:
 
             ### FIX ME ### should abort cert and try to clear on error
 
-            self.propagation_delay(challenges)
+            error, errata_list = self.propagation_delay(challenges)
 
             # for a case where you want certificates for *.example.com and example.com
             # you have to create both auth records AND then respond to the challenge.
@@ -762,6 +575,9 @@ class Client:
             ### TO DO ### this is the obfuscated timeout loop.  Clean this mess up!
             ### # # # ### it also keeps trying even when the auth is failed :-(
 
+            ### FIX? ### shouldn't this be checking the ORDER's status for completion?
+            #            that is at least the most frugal of queries approach...
+
             for chal in challenges:
                 # Before sending a CSR, we need to make sure the server has completed the
                 # validation for all the authorizations
@@ -789,56 +605,57 @@ class Client:
         while True:
             yield cur_time
 
-    def propagation_delay(self, challenges):
+    def propagation_delay(self, challenges: ChalListType) -> Tuple[str, ErrataListType]:
         """
         Wait for the challenges to propagate through the service.
 
-        SHOULD return success or failure... TBD.
+        Returns (error: str, errata_list)
+        * ("", []) is complete success
+        * ("timeout", [...]) list contains challenges that weren't ready
+        * ("failure", [...]) list contains both failed and not-yet-ready challenges
 
-        This process is controlled by two attributes and one method on the provider.
-        unpropagated() is called to check whether the listed challenges are thought
-        to be ready to be validated by the ACME server.  If it fails, propagation_delay
-        will sleep and retry until it times out.  The attributes on the provider are
-        prop_timeout and prop_sleep_times.  See docs/unpropagated.md for the details.
+        See docs/unpropagated.md for the details.
         """
 
+        if self.provider.prop_delay:
+            time.sleep(self.provider.prop_delay)
+
         if self.provider.prop_timeout:
             unready = challenges
             end_time = time.time() + self.provider.prop_timeout
-            next_sleep = self.sleep_iter()
+            sleep_time = self.sleep_iter()
             num_checks = 0
+
             while unready:
                 errata = self.provider.unpropagated(unready)
                 num_checks += 1
 
-                ### FIX ME ### if any errata have status "failed", exit with error
+                # right idea, but details aren't yet nailed down?
+                # failed = [e for e in errata if e['status'].startswith("FAIL")]
+
                 if errata:
                     poll_time = time.time()
                     # intentional: do an "extra" check rather than running short
                     if end_time < poll_time:
                         break
                     # wait a while to let more propagation happen
-                    time.sleep(next(next_sleep))
+                    time.sleep(next(sleep_time))
 
                 unready = [err[2] for err in errata]
 
-            ### FIX ME ### might be good for mock tests, but really should try to clear, eh?
             if unready:
+                ### FIX ME ### might be good for mock tests, but really should try to clear, eh?
+                # return ("timeout", unready)
                 raise RuntimeError(
                     "propagation_delay: time out after %s probes: %s" % (num_checks, unready)
                 )
 
+        return ("", [])
+
     def cert(self):
-        """
-        convenience method to get a certificate without much hassle
-        """
+        self.logger.warning("DEPRECATED: Client.cert is deprecated as of 0.8.4")
         return self.get_certificate()
 
     def renew(self):
-        """
-        renews a certificate.
-        A renewal is actually just getting a new certificate.
-        An issuance request counts as a renewal if it contains the exact same set of hostnames as a previously issued certificate.
-            https://letsencrypt.org/docs/rate-limits/
-        """
+        self.logger.warning("DEPRECATED: Client.renew is deprecated as of 0.8.4")
         return self.cert()
