@@ -18,6 +18,7 @@ class CloudFlareDns(common.BaseDns):
         CLOUDFLARE_API_KEY=None,
         CLOUDFLARE_API_BASE_URL="https://api.cloudflare.com/client/v4/",
         CLOUDFLARE_TOKEN=None,
+        **kwargs,
     ):
         self.CLOUDFLARE_DNS_ZONE_ID = None
         self.CLOUDFLARE_EMAIL = CLOUDFLARE_EMAIL
@@ -40,7 +41,7 @@ class CloudFlareDns(common.BaseDns):
                 "Error initializing Cloudflare DNS adapter. Pass either email and API key or a token."
             )
 
-        super(CloudFlareDns, self).__init__()
+        super().__init__(**kwargs)
 
     def find_dns_zone(self, domain_name):
         self.logger.debug("find_dns_zone")
@@ -64,10 +65,11 @@ class CloudFlareDns(common.BaseDns):
                 setattr(self, "CLOUDFLARE_DNS_ZONE_ID", i["id"])
         if isinstance(self.CLOUDFLARE_DNS_ZONE_ID, type(None)):
             raise ValueError(
-                "Error unable to get DNS zone for domain_name={domain_name}: status_code={status_code} response={response}".format(
-                    domain_name=domain_name,
-                    status_code=find_dns_zone_response.status_code,
-                    response=log_response(find_dns_zone_response),
+                "No DNS zone for %s: status = %s, response=%s"
+                % (
+                    domain_name,
+                    find_dns_zone_response.status_code,
+                    log_response(find_dns_zone_response),
                 )
             )
 
