@@ -11,7 +11,9 @@ class DNSPodDns(common.BaseDns):
 
     dns_provider_name = "dnspod"
 
-    def __init__(self, DNSPOD_ID, DNSPOD_API_KEY, DNSPOD_API_BASE_URL="https://dnsapi.cn/"):
+    def __init__(
+        self, DNSPOD_ID, DNSPOD_API_KEY, DNSPOD_API_BASE_URL="https://dnsapi.cn/", **kwargs
+    ):
         self.DNSPOD_ID = DNSPOD_ID
         self.DNSPOD_API_KEY = DNSPOD_API_KEY
         self.DNSPOD_API_BASE_URL = DNSPOD_API_BASE_URL
@@ -22,14 +24,16 @@ class DNSPodDns(common.BaseDns):
             self.DNSPOD_API_BASE_URL = DNSPOD_API_BASE_URL + "/"
         else:
             self.DNSPOD_API_BASE_URL = DNSPOD_API_BASE_URL
-        super(DNSPodDns, self).__init__()
+        super().__init__(**kwargs)
 
     def create_dns_record(self, domain_name, domain_dns_value):
         self.logger.info("create_dns_record")
+
+        ### FIX ME   ### domain is exactly last two parts (service has list of zones API?)
+        ### YUCK     ### Odd, unpythonic implementation
+        ### REFACTOR ### duplicated code (here and delete_dns_record)
+
         subd = ""
-
-        ### FIX ME ### domain is exactly last two parts (no bbc.co.uk eg.)
-
         if domain_name.count(".") != 1:  # not top level domain
             pos = domain_name.rfind(".", 0, domain_name.rfind("."))
             subd = domain_name[:pos]
@@ -69,10 +73,8 @@ class DNSPodDns(common.BaseDns):
 
     def delete_dns_record(self, domain_name, domain_dns_value):
         self.logger.info("delete_dns_record")
+
         subd = ""
-
-        ### FIX ME ### domain is exactly last two parts (no bbc.co.uk eg.)
-
         if domain_name.count(".") != 1:  # not top level domain
             pos = domain_name.rfind(".", 0, domain_name.rfind("."))
             subd = domain_name[:pos]
