@@ -16,12 +16,15 @@ class Route53Dns(common.BaseDns):
     connect_timeout = 30
     read_timeout = 30
 
-    def __init__(self, access_key_id=None, secret_access_key=None):
+    def __init__(self, access_key_id=None, secret_access_key=None, client=None, **kwargs):
         if not route53_dependencies:
             raise ImportError(
                 """You need to install Route53Dns dependencies. run; pip3 install sewer[route53]"""
             )
 
+        if (access_key_id or secret_access_key) and client:
+            raise RuntimeError("Pass keys OR preconfigured client, not both")
+
         self.aws_config = Config(
             connect_timeout=self.connect_timeout, read_timeout=self.read_timeout
         )
@@ -33,6 +36,9 @@ class Route53Dns(common.BaseDns):
                 aws_secret_access_key=secret_access_key,
                 config=self.aws_config,
             )
+        elif client:
+            # Use the client passed in from the caller.
+            self.r53 = client
         else:
             # let boto3 find credential
             # https://boto3.readthedocs.io/en/latest/guide/configuration.html#best-practices-for-configuring-credentials
@@ -40,7 +46,7 @@ class Route53Dns(common.BaseDns):
 
         self._resource_records = collections.defaultdict(list)
 
-        super(Route53Dns, self).__init__()
+        super().__init__(**kwargs)
 
     def create_dns_record(self, domain_name, domain_dns_value):
         challenge_domain = "_acme-challenge" + "." + domain_name + "."
